// 
//	Holiday Screen Saver for BeOS
//	Inspired by Christmas Trees
//	by Larry B. Daniel <larry@america.net>
//	My part of the code is Public Domain!
//	Use it, Sell it, Eat Mo' Code!
//	

#include <Window.h>
#include <View.h>
#include <TextView.h>
#include <StringView.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include "holiday.h"

//
// A bit of Global State
//
static bool go=false;
static bool prev=false;
static bool restore=false;
static BView * rv;
//
// I N I T I A L I Z E
//	
void module_initialize(void *settings, long settings_size) {
	#pragma unused (settings, settings_size)
}
//
// C L E A N U P
//	
void module_cleanup(void **settings, long *settings_size) {
	#pragma unused(settings)
	#pragma unused(settings_size)
}
//
// S T A R T   S A V I N G
//	
void module_start_saving(BView *v) {
	
	if (go) { 
		module_stop_saving();
		prev = false;
		restore = true;
		rv = glv;
	}
	glv = v;
	v->Window()->Lock();
	v->Window()->Show();
	v->Window()->Unlock();
	go = true;
	saver_thread = spawn_thread((thread_func)saver_function, "Friday Saver thread", B_LOW_PRIORITY, (void*)v);
	resume_thread(saver_thread);
	}
//
// S T O P   S A V I N G
//	
void module_stop_saving() {
	go = false;
	long result;
	wait_for_thread(saver_thread, &result);
	if (restore) {
		restore=false;
		prev = true;
		module_start_saving(rv);
	}
}
//
// S T A R T   C O N F I G
//	
void module_start_config(
	BView *inView)
{
	if (!inView->Window()->Lock())
		return;
		
	BRect frame;
	
	frame = inView->Bounds();
	frame.top += 10.0;
	frame.left += 10.0;
	frame.bottom = frame.top + 12.0;
	BStringView *view = 
		new BStringView(frame, B_EMPTY_STRING, "Holiday");
	inView->AddChild(view);
	view->SetViewColor(inView->ViewColor());
	view->SetFont(be_bold_font);
	
	frame = inView->Bounds();
	frame.top += 25.0;
	frame.bottom = frame.top + 40.0;
	frame.left += 20.0;
	frame.right -= 20.0;
	BRect textArea = frame;
	textArea.OffsetTo(B_ORIGIN);
	BTextView *caption = new BTextView(frame, B_EMPTY_STRING, textArea,
						   			   B_FOLLOW_ALL, B_WILL_DRAW);
	caption->SetText("Inspired by Christmas Trees\n" 
					 "by Larry Daniel, 1997,1998\n"
					 "<larry@america.net>");
	inView->AddChild(caption);
	caption->SetViewColor(inView->ViewColor());
	caption->SetDrawingMode(B_OP_OVER);
	caption->SetFont(be_plain_font);
	caption->MakeEditable(FALSE);
	caption->MakeSelectable(FALSE);
	caption->SetWordWrap(TRUE);	
	
	BRect previewFrame = inView->Bounds();
	previewFrame.top = frame.bottom + 10.0;
	previewFrame.left += 10.0;
	previewFrame.right -= 10.0;
	previewFrame.bottom -= 10.0;
	BView *preview = new BView(previewFrame, B_EMPTY_STRING, 
							   B_FOLLOW_ALL, B_WILL_DRAW);
	inView->AddChild(preview);		
	preview->SetViewColor(0,0,0);
	preview->SetLowColor(0,0,0);
	preview->Invalidate();
	
	inView->Window()->UpdateIfNeeded();
	inView->Window()->Unlock();
	
	prev = true;
	module_start_saving(preview);
}
//
// S T O P   C O N F I G
//	
void module_stop_config(){
	module_stop_saving();
	prev=false;
}
//
// S A V E R   F U N C T I O N
//	
long saver_function(BView * v) {
	
	// Data
	v->Window()->Lock();
	BRect r = v->Bounds();
	
	v->SetHighColor(0,0,0);
	v->FillRect(glv->Bounds());
	v->MovePenTo(50.0,50.0);
	v->Sync();
	if (prev) {
		v->SetPenSize(2.0);		
	}else {
		v->SetPenSize(4.0);		
	}
	v->Window()->Unlock();
	
	float w = r.Width();
	float h = r.Height();
	
	int i=0;
	BPoint x(0.0,0.0), y(w,h);
	BPoint d(0.0,1.0), u(0.0,-1.0);
	unsigned char red=0, green=0, blue=0, alpha=0;
	float pi2 = 2.0* PI;
	float cred, cgreen, cblue;
	float yx, yy;
	srand(clock());
	v->Window()->Lock();
	while (go) {   
		red=(double(rand())/RAND_MAX)*255; 
		green=(double(rand())/RAND_MAX)*255;
		blue=(double(rand())/RAND_MAX)*255;
		
		//red ++;
		//green --;
		//blue += (255-red);	
		// scale colors to 2pi
		cred = pi2/255.0 * red;
		cgreen = pi2/255.0 * green;
		cblue = pi2/255.0 * blue;
		// screen scale factors 			
		
		if (i%200 == 0) {
			yx = (double(rand())/RAND_MAX)*w/1.3 + w/8;
			yy = (double(rand())/RAND_MAX)*h/1.3 + h/8;
		}
		++i;
		x = BPoint(sin(cgreen)*sin(cred)*sin(cblue)*w/2+yx,
		  		   sin(cgreen)*cos(cred)*cos(cblue)*h/2+yy);
		y = BPoint(yx, yy);
//		y = BPoint(sin(cgreen)*cos(cblue)*sin(cred)*w/2+w/2.0,
//			 	   sin(cgreen)*sin(cblue)*sin(cred)*h/2+h/2.0);
		v->SetHighColor(red,green,blue,alpha);
		v->StrokeLine(x,y);
		if (i%10000==0) {
			v->SetHighColor(0,0,0);
			v->FillRect(glv->Bounds());
			}
		if (prev ) {
			v->Window()->Unlock();
			snooze(1000);
			v->Window()->Lock();
			}
	}
	v->Window()->Unlock();
	
	// Need a Return Value
	return 0;
}
